"""All helpers for the context."""


def does_not_exist_msg(title):
    """
    Make the message when something does not exist.

    Parameters
    ----------
    title : str
        Title to set to the message.

    Returns
    -------
    str
        Message with a title.
    """
    return '%s matching query does not exist.' % title



"""All models for the extension."""

from django.conf import settings
from django.dispatch import receiver
from django.db import models

from model_utils.models import StatusModel, TimeStampedModel

from geokey.projects.models import Project

from .base import STATUS, FORMAT
from .managers import WebResourceManager


class WebResource(StatusModel, TimeStampedModel):
    """Store a single web resource."""

    STATUS = STATUS
    FORMAT = FORMAT

    name = models.CharField(max_length=100)
    description = models.TextField(null=True, blank=True)
    data_format = models.CharField(max_length=10, null=False, choices=FORMAT)
    url = models.URLField(max_length=250)
    order = models.IntegerField(default=0)
    colour = models.TextField(default='#0033ff')
    symbol = models.ImageField(
        upload_to='webresources/symbols',
        max_length=500,
        null=True,
        blank=True
    )

    project = models.ForeignKey(
        'projects.Project',
        related_name='webresources'
    )
    creator = models.ForeignKey(settings.AUTH_USER_MODEL)

    objects = WebResourceManager()

    class Meta:
        """Model meta."""

        ordering = ['order']

    def delete(self):
        """Delete the web resource by setting its status to `deleted`."""
        self.status = self.STATUS.deleted
        self.save()


@receiver(models.signals.post_save, sender=Project)
def post_save_project(sender, instance, **kwargs):
    """Remove associated web resources when the project gets deleted."""
    if instance.status == 'deleted':
        WebResource.objects.filter(project=instance).delete()



"""All URLs for the extension."""

from django.conf.urls import url

from .views import (
    IndexPage,
    AllWebResourcesPage,
    AddWebResourcePage,
    SingleWebResourcePage,
    RemoveWebResourcePage,
    ReorderWebResourcesAjax,
    UpdateWebResourceAjax,
    AllWebResourcesAPI,
    SingleWebResourceAPI
)


urlpatterns = [
    # ###########################
    # ADMIN PAGES
    # ###########################

    url(
        r'^admin/webresources/$',
        IndexPage.as_view(),
        name='index'),
    url(
        r'^admin/projects/(?P<project_id>[0-9]+)/'
        r'webresources/$',
        AllWebResourcesPage.as_view(),
        name='all_webresources'),
    url(
        r'^admin/projects/(?P<project_id>[0-9]+)/'
        r'webresources/add/$',
        AddWebResourcePage.as_view(),
        name='webresource_add'),
    url(
        r'^admin/projects/(?P<project_id>[0-9]+)/'
        r'webresources/(?P<webresource_id>[0-9]+)/$',
        SingleWebResourcePage.as_view(),
        name='single_webresource'),
    url(
        r'^admin/projects/(?P<project_id>[0-9]+)/'
        r'webresources/(?P<webresource_id>[0-9]+)/remove/$',
        RemoveWebResourcePage.as_view(),
        name='webresource_remove'),

    # ###########################
    # ADMIN AJAX
    # ###########################

    url(
        r'^ajax/projects/(?P<project_id>[0-9]+)/'
        r'webresources/reorder/$',
        ReorderWebResourcesAjax.as_view(),
        name='ajax_webresources_reorder'),
    url(
        r'^ajax/projects/(?P<project_id>[0-9]+)/'
        r'webresources/(?P<webresource_id>[0-9]+)/$',
        UpdateWebResourceAjax.as_view(),
        name='ajax_webresource_update'),

    # ###########################
    # PUBLIC API
    # ###########################

    url(
        r'^api/projects/(?P<project_id>[0-9]+)/'
        r'webresources/$',
        AllWebResourcesAPI.as_view(),
        name='api_all_webresources'),
    url(
        r'^api/projects/(?P<project_id>[0-9]+)/'
        r'webresources/(?P<webresource_id>[0-9]+)/$',
        SingleWebResourceAPI.as_view(),
        name='api_single_webresource')
]



"""All views for the extension."""

# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.views.generic import CreateView, FormView, TemplateView
from django.shortcuts import redirect
from django.utils.safestring import mark_safe
from django.contrib import messages

from rest_framework import status
from rest_framework.views import APIView
from rest_framework.response import Response

from braces.views import LoginRequiredMixin

from geokey.core.decorators import handle_exceptions_for_ajax
from geokey.projects.models import Project
from geokey.projects.views import ProjectContext

from .helpers.context_helpers import does_not_exist_msg
from .base import STATUS, FORMAT
from .models import WebResource
from .forms import WebResourceForm
from .serializers import WebResourceSerializer


# ###########################
# ADMIN PAGES
# ###########################

class IndexPage(LoginRequiredMixin, TemplateView):
    """Main index page."""

    template_name = 'wr_index.html'

    def get_context_data(self, *args, **kwargs):
        """
        GET method for the template.

        Return the context to render the view. Overwrite the method by adding
        all projects (where user is an administrator) to the context.

        Returns
        -------
        dict
            Context.
        """
        projects = Project.objects.filter(admins=self.request.user)

        return super(IndexPage, self).get_context_data(
            projects=projects,
            *args,
            **kwargs
        )


class AllWebResourcesPage(LoginRequiredMixin, ProjectContext, TemplateView):
    """All web resources page."""

    template_name = 'wr_all_webresources.html'


class AddWebResourcePage(LoginRequiredMixin, ProjectContext, CreateView):
    """Add new web resource page."""

    template_name = 'wr_add_webresource.html'
    form_class = WebResourceForm

    def get_context_data(self, *args, **kwargs):
        """
        GET method for the template.

        Return the context to render the view. Overwrite the method by adding
        project ID and available data formats to the context.

        Returns
        -------
        dict
            Context.
        """
        project_id = self.kwargs['project_id']

        return super(AddWebResourcePage, self).get_context_data(
            project_id,
            data_formats=FORMAT,
            *args,
            **kwargs
        )

    def form_valid(self, form):
        """
        Add web resource when form data is valid.

        Parameters
        ----------
        form : geokey_webresource.forms.WebResourceForm
            Represents the user input.

        Returns
        -------
        django.http.HttpResponse
            Rendered template.
        """
        context = self.get_context_data(form=form)
        project = context.get('project')

        if project:
            if project.islocked:
                messages.error(
                    self.request,
                    'The project is locked. New web resources cannot be added.'
                )
            else:
                form.instance.project = project
                form.instance.creator = self.request.user

                add_another_url = reverse(
                    'geokey_webresources:webresource_add',
                    kwargs={
                        'project_id': project.id
                    }
                )
                messages.success(
                    self.request,
                    mark_safe(
                        'The web resource has been added. <a href="%s">Add '
                        'another web resource.</a>' % add_another_url
                    )
                )
                return super(AddWebResourcePage, self).form_valid(form)

        return self.render_to_response(context)

    def form_invalid(self, form):
        """
        Display an error message when form data is invalid.

        Parameters
        ----------
        form : geokey_webresource.forms.WebResourceForm
            Represents the user input.

        Returns
        -------
        dict
            Context.
        """
        messages.error(self.request, 'An error occurred.')
        return self.render_to_response(self.get_context_data(form=form))

    def get_success_url(self):
        """
        Set URL redirection when web resource created successfully.

        Returns
        -------
        str
            URL for redirection.
        """
        return reverse(
            'geokey_webresources:all_webresources',
            kwargs={
                'project_id': self.kwargs['project_id']
            }
        )


class WebResourceContext(LoginRequiredMixin, ProjectContext):
    """Get web resource mixin."""

    def get_context_data(self, project_id, webresource_id, *args, **kwargs):
        """
        GET method for the template.

        Return the context to render the view. Overwrite the method by adding
        a web resource and available status types & data formats to the
        context.

        Parameters
        ----------
        project_id : int
            Identifies the project in the database.
        webresource_id : int
            Identifies the web resource in the database.

        Returns
        -------
        dict
            Context.
        """
        context = super(WebResourceContext, self).get_context_data(
            project_id,
            *args,
            **kwargs
        )

        context['status_types'] = STATUS
        context['data_formats'] = FORMAT

        try:
            context['webresource'] = WebResource.objects.get(
                pk=webresource_id,
                project=context.get('project')
            )

            return context
        except WebResource.DoesNotExist:
            return {
                'error': 'Not found.',
                'error_description': does_not_exist_msg('Web resource')
            }


class SingleWebResourcePage(WebResourceContext, FormView):
    """Single web resource page."""

    template_name = 'wr_single_webresource.html'
    form_class = WebResourceForm

    def get_object(self):
        """
        Get and return web resource object.

        Returns
        -------
        geokey_webresource.models.WebResource
            Web resource object.
        """
        try:
            return WebResource.objects.get(
                pk=self.kwargs['webresource_id']
            )
        except WebResource.DoesNotExist:
            return None

    def get_context_data(self, *args, **kwargs):
        """
        GET method for the template.

        Return the context to render the view. Overwrite the method by adding
        project ID and web resource ID to the context.

        Returns
        -------
        dict
            Context.
        """
        project_id = self.kwargs['project_id']
        webresource_id = self.kwargs['webresource_id']

        return super(SingleWebResourcePage, self).get_context_data(
            project_id,
            webresource_id,
            *args,
            **kwargs
        )

    def get_form(self, form_class):
        """Attach instance object to form data."""
        return form_class(instance=self.get_object(), **self.get_form_kwargs())

    def form_valid(self, form):
        """
        Update web resource when form data is valid.

        Parameters
        ----------
        form : geokey_webresource.forms.WebResourceForm
            Represents the user input.

        Returns
        -------
        django.http.HttpResponse
            Rendered template.
        """
        context = self.get_context_data(form=form)
        project = context.get('project')

        if project:
            if project.islocked:
                messages.error(
                    self.request,
                    'The project is locked. Web resources cannot be updated.'
                )
            else:
                if self.request.POST.get('clear-symbol') == 'true':
                    form.instance.symbol = None
                form.save()

                messages.success(
                    self.request,
                    mark_safe('The web resource has been updated.')
                )
                return super(SingleWebResourcePage, self).form_valid(form)

        return self.render_to_response(context)

    def form_invalid(self, form):
        """
        Display an error message when form data is invalid.

        Parameters
        ----------
        form : geokey_webresource.forms.WebResourceForm
            Represents the user input.

        Returns
        -------
        dict
            Context.
        """
        messages.error(self.request, 'An error occurred.')
        return self.render_to_response(self.get_context_data(form=form))

    def get_success_url(self):
        """
        Set URL redirection when web resource updated successfully.

        Returns
        -------
        str
            URL for redirection.
        """
        return reverse(
            'geokey_webresources:all_webresources',
            kwargs={
                'project_id': self.kwargs['project_id']
            }
        )


class RemoveWebResourcePage(WebResourceContext, TemplateView):
    """Remove web resource page."""

    template_name = 'base.html'

    def get(self, request, project_id, webresource_id):
        """
        GET method for removing web resource.

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        webresource_id : int
            Identifies the web resource in the database.

        Returns
        -------
        django.http.HttpResponseRedirect
            Redirects to all web resources if web resource is removed, single
            web resource page if project is locked.
        django.http.HttpResponse
            Rendered template if project or web resource does not exist.
        """
        context = self.get_context_data(project_id, webresource_id)
        webresource = context.get('webresource')

        if webresource:
            if webresource.project.islocked:
                messages.error(
                    request,
                    'The project is locked. Web resource cannot be removed.'
                )
                return redirect(
                    'geokey_webresources:single_webresource',
                    project_id=project_id,
                    webresource_id=webresource_id
                )
            else:
                webresource.delete()
                messages.success(
                    request,
                    'The web resource has been removed.'
                )
                return redirect(
                    'geokey_webresources:all_webresources',
                    project_id=project_id
                )

        return self.render_to_response(context)


# ###########################
# ADMIN AJAX
# ###########################

class ReorderWebResourcesAjax(APIView):
    """Reorder web resources via Ajax."""

    @handle_exceptions_for_ajax
    def post(self, request, project_id):
        """
        POST method for reordering web resources.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            Identifies the project in the database.

        Returns
        -------
        rest_framework.response.Response
            Response to the request.
        """
        project = Project.objects.as_admin(request.user, project_id)

        if project.islocked:
            return Response(
                {'error': 'Project is locked.'},
                status=status.HTTP_403_FORBIDDEN
            )
        elif not project.webresources.exists():
            return Response(
                {'error': 'Project has no web resources.'},
                status=status.HTTP_404_NOT_FOUND
            )

        try:
            webresources = []

            for order, webresource_id in enumerate(request.data.get('order')):
                webresource = project.webresources.get(pk=webresource_id)
                webresource.order = order
                webresources.append(webresource)

            for webresource in webresources:
                webresource.save()

            serializer = WebResourceSerializer(
                project.webresources,
                many=True
            )
            return Response(serializer.data)
        except WebResource.DoesNotExist:
            return Response(
                {'error': 'One or more web resources were not found.'},
                status=status.HTTP_400_BAD_REQUEST
            )


class UpdateWebResourceAjax(APIView):
    """Update web resource via Ajax."""

    @handle_exceptions_for_ajax
    def put(self, request, project_id, webresource_id):
        """
        PUT method for updating web resource.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        webresource_id : int
            Identifies the web resource in the database.

        Returns
        -------
        rest_framework.response.Response
            Response to the request.
        """
        project = Project.objects.as_admin(request.user, project_id)

        if project.islocked:
            return Response(
                {'error': 'Project is locked.'},
                status=status.HTTP_403_FORBIDDEN
            )

        try:
            webresource = project.webresources.get(pk=webresource_id)
            serializer = WebResourceSerializer(
                webresource,
                data=request.data,
                partial=True,
                fields=('id', 'status')
            )

            if serializer.is_valid():
                serializer.save()
                return Response(serializer.data)
            else:
                return Response(
                    serializer.errors,
                    status=status.HTTP_400_BAD_REQUEST
                )
        except WebResource.DoesNotExist, error:
            return Response(
                {'error': str(error)},
                status=status.HTTP_404_NOT_FOUND
            )


# ###########################
# PUBLIC API
# ###########################

class AllWebResourcesAPI(APIView):
    """All web resources via API."""

    @handle_exceptions_for_ajax
    def get(self, request, project_id):
        """
        GET method for all web resources of a project.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            Identifies the project in the database.

        Returns
        -------
        rest_framework.response.Response
            Response to the request.
        """
        project = Project.objects.get_single(request.user, project_id)
        serializer = WebResourceSerializer(
            project.webresources.filter(status=STATUS.active),
            many=True
        )
        return Response(serializer.data)


class SingleWebResourceAPI(APIView):
    """Single web resource via API."""

    @handle_exceptions_for_ajax
    def get(self, request, project_id, webresource_id):
        """
        GET method for a single web resource of a project.

        Only active web resources are returned to anyone who has access to the
        project.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            Identifies the project in the database.
        webresource_id : int
            Identifies the web resource in the database.

        Returns
        -------
        rest_framework.response.Response
            Response to the request.
        """
        project = Project.objects.get_single(request.user, project_id)

        try:
            webresource = project.webresources.get(
                pk=webresource_id,
                status=STATUS.active
            )
            serializer = WebResourceSerializer(webresource)
            return Response(serializer.data)
        except WebResource.DoesNotExist:
            return Response(
                {'error': 'Web resource not found.'},
                status=status.HTTP_404_NOT_FOUND
            )



"""All forms for the extension."""

from django.forms import ModelForm

from .models import WebResource


class WebResourceForm(ModelForm):
    """Form for a single web resource."""

    class Meta:
        """Form meta."""

        model = WebResource
        fields = ('name', 'description', 'data_format', 'url', 'colour',
                  'symbol')



"""All managers for the extension."""

from django.db import models

from .base import STATUS


class WebResourceManager(models.Manager):
    """Manage a single web resource."""

    def get_queryset(self):
        """
        Return all web resources.

        Returns
        -------
        django.db.models.Queryset
            All web resources, excluding deleted.
        """
        return super(
            WebResourceManager,
            self
        ).get_queryset().exclude(status=STATUS.deleted)



"""All serializers for the extension."""

from rest_framework.serializers import SerializerMethodField

from geokey.core.serializers import FieldSelectorSerializer

from .models import WebResource


class WebResourceSerializer(FieldSelectorSerializer):
    """Serializer for a web resource."""

    symbol = SerializerMethodField()

    def get_symbol(self, webresource):
        """
        Get URL of a symbol.

        Parameters
        ----------
        webresource : geokey_webresources.models.WebResource
            Web resource that is being serialised.
        """
        if webresource.symbol:
            return webresource.symbol.url

        return None

    class Meta:
        """Serializer meta."""

        model = WebResource
        fields = ('id', 'status', 'name', 'description', 'created', 'modified',
                  'data_format', 'url', 'colour', 'symbol')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('projects', '0007_auto_20160122_1409'),
    ]

    operations = [
        migrations.CreateModel(
            name='WebResource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('status', model_utils.fields.StatusField(choices=[(b'active', b'active'), (b'inactive', b'inactive'), (b'deleted', b'deleted')], default=b'active', max_length=100, no_check_for_status=True, verbose_name='status')),
                ('status_changed', model_utils.fields.MonitorField(default=django.utils.timezone.now, monitor='status', verbose_name='status changed')),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(blank=True, null=True)),
                ('data_format', models.CharField(choices=[(b'GeoJSON', b'GeoJSON'), (b'KML', b'KML'), (b'GPX', b'GPX')], max_length=10)),
                ('url', models.URLField(max_length=250)),
                ('order', models.IntegerField(default=0)),
                ('colour', models.TextField(default=b'#0033ff')),
                ('symbol', models.ImageField(blank=True, max_length=500, null=True, upload_to=b'webresources/symbols')),
                ('creator', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
                ('project', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='webresources', to='projects.Project')),
            ],
            options={
                'ordering': ['order'],
            },
        ),
    ]



#!/usr/bin/env python

"""GeoKey extension for adding web resources to projects."""

from os.path import join
from setuptools import setup, find_packages


name = 'geokey-webresources'
version = __import__(name.replace('-', '_')).__version__
repository = join('https://github.com/ExCiteS', name)

setup(
    name=name,
    version=version,
    description='Add web resources to GeoKey projects',
    url=repository,
    download_url=join(repository, 'tarball', version),
    author='ExCiteS',
    author_email='excites@ucl.ac.uk',
    license='MIT',
    packages=find_packages(exclude=['*.tests', '*.tests.*', 'tests.*']),
    include_package_data=True,
    install_requires=[],
)



"""Main initialisation for extension."""

VERSION = (0, 1, 1)
__version__ = '.'.join(map(str, VERSION))


try:
    from geokey.extensions.base import register

    register(
        'geokey_webresources',
        'Web Resources',
        display_admin=True,
        superuser=False,
        version=__version__
    )
except BaseException:
    print 'Please install GeoKey first'
